#include "controlwindow.hpp"

ControlWindow::ControlWindow(QWidget* parent)
: QWidget(parent)
{
    layout = new QVBoxLayout(this);

    //
    // Einrichten saemtlicher UI-Elemente des Einstellungs-Fensters.
    //
    layout->addWidget(createMandelbrotGroupBox());
    layout->addWidget(createMandelbarGroupBox());
    layout->addWidget(createMultibrotGroupBox());
    layout->addWidget(createJuliaGroupBox());
    layout->addWidget(createPositionGroupBox());
    layout->addWidget(createZoomGroupBox());
    layout->addWidget(createPrecisionGroupBox());
    layout->addWidget(createColorGroupBox());
    layout->addWidget(createAlgorithmGroupBox());
    layout->addWidget(createRestoreGroupBox());
    layout->addWidget(createInformationGroupBox());

    setLayout(layout);
    setFixedSize(sizeHint());
    setWindowTitle(tr("Einstellungen"));
    setWindowIcon(QIcon(":/hsrmlogo.png"));
}

void ControlWindow::closeEvent(QCloseEvent *event)
{
    //
    // Wenn eines der Fenster geschlossen wird, beendet sich die Anwendung.
    //
    event->accept();
    emit closeAllWindows();
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox "Mandelbrot-Menge".
//
QGroupBox* ControlWindow::createMandelbrotGroupBox()
{
    mandelbrotGroupBox = new QGroupBox(tr("Mandelbrot-Menge"), this);

    renderMandelbrotButton = new QPushButton(tr("Berechnen und anzeigen"), this);

    helpMandelbrotButton = new QPushButton(tr("?"), this);
    helpMandelbrotButton->setFixedWidth(helpMandelbrotButton->sizeHint().width()/2);

    mandelbrotLayout = new QVBoxLayout;
    mandelbrotSubLayout = new QHBoxLayout;

    mandelbrotSubLayout->addWidget(renderMandelbrotButton);
    mandelbrotSubLayout->addWidget(helpMandelbrotButton);
    mandelbrotLayout->addLayout(mandelbrotSubLayout);

    mandelbrotGroupBox->setLayout(mandelbrotLayout);

    return mandelbrotGroupBox;
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox "Mandelbar-Menge".
//
QGroupBox* ControlWindow::createMandelbarGroupBox()
{
    mandelbarGroupBox = new QGroupBox(tr("Mandelbar-Menge"), this);

    renderMandelbarButton = new QPushButton(tr("Berechnen und anzeigen"), this);

    helpMandelbarButton = new QPushButton(tr("?"), this);
    helpMandelbarButton->setFixedWidth(helpMandelbarButton->sizeHint().width()/2);

    mandelbarLayout = new QVBoxLayout;
    mandelbarSubLayout = new QHBoxLayout;

    mandelbarSubLayout->addWidget(renderMandelbarButton);
    mandelbarSubLayout->addWidget(helpMandelbarButton);
    mandelbarLayout->addLayout(mandelbarSubLayout);

    mandelbarGroupBox->setLayout(mandelbarLayout);

    return mandelbarGroupBox;
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox "Multibrot-Menge".
//
QGroupBox* ControlWindow::createMultibrotGroupBox()
{
    multibrotGroupBox = new QGroupBox(tr("Multibrot-Menge"), this);

    exponentRangeLabel = new QLabel(tr("[2,00; 100,00]"), this);

    exponentParamLabel = new QLabel(tr("Exponent:"), this);

    multibrotSpinBox = new QDoubleSpinBox(this);
    multibrotSpinBox->setValue(2.00);
    multibrotSpinBox->setRange(2.00, 100.00);
    multibrotSpinBox->setSingleStep(0.01);
    multibrotSpinBox->setFixedSize(multibrotSpinBox->sizeHint());

    renderMultibrotButton = new QPushButton(tr("Berechnen und anzeigen"), this);

    helpMultibrotButton = new QPushButton(tr("?"), this);
    helpMultibrotButton->setFixedWidth(helpMultibrotButton->sizeHint().width()/2);

    multibrotLayout = new QVBoxLayout;
    multibrotSubLayout1 = new QHBoxLayout;
    multibrotSubLayout2 = new QHBoxLayout;

    multibrotSubLayout1->addWidget(exponentRangeLabel);
    multibrotSubLayout1->addWidget(exponentParamLabel);
    multibrotSubLayout1->addWidget(multibrotSpinBox);
    multibrotLayout->addLayout(multibrotSubLayout1);
    multibrotSubLayout2->addWidget(renderMultibrotButton);
    multibrotSubLayout2->addWidget(helpMultibrotButton);
    multibrotLayout->addLayout(multibrotSubLayout2);

    multibrotGroupBox->setLayout(multibrotLayout);

    return multibrotGroupBox;
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox "Julia-Menge".
//
QGroupBox* ControlWindow::createJuliaGroupBox()
{
    juliaGroupBox = new QGroupBox(tr("Julia-Menge"), this);

    realRangeLabel = new QLabel(tr("[-1,00; 1,00]"), this);

    realParamLabel = new QLabel(tr("Realteil:"), this);

    juliaRealSpinBox = new QDoubleSpinBox(this);
    juliaRealSpinBox->setRange(-1.00, 1.00);
    juliaRealSpinBox->setSingleStep(0.01);
    juliaRealSpinBox->setFixedSize(multibrotSpinBox->size());

    imagRangeLabel = new QLabel(tr("[-1,00; 1,00]"), this);

    imagParamLabel = new QLabel(tr("Imaginaerteil:"), this);

    juliaImagSpinBox = new QDoubleSpinBox(this);
    juliaImagSpinBox->setRange(-1.00, 1.00);
    juliaImagSpinBox->setSingleStep(0.01);
    juliaImagSpinBox->setFixedSize(multibrotSpinBox->size());

    renderJuliaButton = new QPushButton(tr("Berechnen und anzeigen"), this);

    helpJuliaButton = new QPushButton(tr("?"), this);
    helpJuliaButton->setFixedWidth(helpJuliaButton->sizeHint().width()/2);

    juliaLayout = new QVBoxLayout;
    juliaSubLayout1 = new QHBoxLayout;
    juliaSubLayout2 = new QHBoxLayout;
    juliaSubLayout3 = new QHBoxLayout;

    juliaSubLayout1->addWidget(realRangeLabel);
    juliaSubLayout1->addWidget(realParamLabel);
    juliaSubLayout1->addWidget(juliaRealSpinBox);
    juliaLayout->addLayout(juliaSubLayout1);
    juliaSubLayout2->addWidget(imagRangeLabel);
    juliaSubLayout2->addWidget(imagParamLabel);
    juliaSubLayout2->addWidget(juliaImagSpinBox);
    juliaLayout->addLayout(juliaSubLayout2);
    juliaSubLayout3->addWidget(renderJuliaButton);
    juliaSubLayout3->addWidget(helpJuliaButton);
    juliaLayout->addLayout(juliaSubLayout3);

    juliaGroupBox->setLayout(juliaLayout);

    return juliaGroupBox;
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox "Mittelpunkt".
//
QGroupBox* ControlWindow::createPositionGroupBox()
{
    positionGroupBox = new QGroupBox(tr("Mittelpunkt"), this);

    xRangeLabel = new QLabel(tr("[-2,00; 2,00]"), this);

    xAxisLabel = new QLabel(tr("X-Achse:"), this);

    xSpinBox = new QDoubleSpinBox(this);
    xSpinBox->setRange(-2.00, 2.00);
    xSpinBox->setSingleStep(0.01);
    xSpinBox->setFixedSize(multibrotSpinBox->size());

    yRangeLabel = new QLabel(tr("[-2,00; 2,00]"), this);

    yAxisLabel = new QLabel(tr("Y-Achse:"), this);

    ySpinBox = new QDoubleSpinBox(this);
    ySpinBox->setRange(-2.00, 2.00);
    ySpinBox->setSingleStep(0.01);
    ySpinBox->setFixedSize(multibrotSpinBox->size());

    positionLayout = new QVBoxLayout;
    positionSubLayout1 = new QHBoxLayout;
    positionSubLayout2 = new QHBoxLayout;

    positionSubLayout1->addWidget(xRangeLabel);
    positionSubLayout1->addWidget(xAxisLabel);
    positionSubLayout1->addWidget(xSpinBox);
    positionLayout->addLayout(positionSubLayout1);
    positionSubLayout2->addWidget(yRangeLabel);
    positionSubLayout2->addWidget(yAxisLabel);
    positionSubLayout2->addWidget(ySpinBox);
    positionLayout->addLayout(positionSubLayout2);

    positionGroupBox->setLayout(positionLayout);

    return positionGroupBox;
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox "Zoomfaktor".
//
QGroupBox* ControlWindow::createZoomGroupBox()
{
    zoomGroupBox = new QGroupBox(tr("Zoomfaktor"), this);

    zoomRangeLabel = new QLabel(tr("[-100; 1 000 000 000]"), this);

    zoomSpinBox = new QSpinBox(this);
    zoomSpinBox->setRange(-100, 1000000000);
    zoomSpinBox->setSingleStep(1);
    zoomSpinBox->setFixedSize(zoomSpinBox->sizeHint());
    zoomSpinBox->setSuffix(tr("%"));

    zoomLayout = new QVBoxLayout;
    zoomSubLayout = new QHBoxLayout;

    zoomSubLayout->addWidget(zoomRangeLabel);
    zoomSubLayout->addWidget(zoomSpinBox);
    zoomLayout->addLayout(zoomSubLayout);

    zoomGroupBox->setLayout(zoomLayout);

    return zoomGroupBox;
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox "Genauigkeit".
//
QGroupBox* ControlWindow::createPrecisionGroupBox()
{
    precisionGroupBox = new QGroupBox(tr("Genauigkeit"), this);

    precisionRangeLabel = new QLabel(tr("[50; 1 000 000]"), this);

    precisionSpinBox = new QSpinBox(this);
    precisionSpinBox->setRange(50, 1000000);
    precisionSpinBox->setSingleStep(1);
    precisionSpinBox->setFixedSize(zoomSpinBox->size());

    precisionLayout = new QVBoxLayout;
    precisionSubLayout = new QHBoxLayout;

    precisionSubLayout->addWidget(precisionRangeLabel);
    precisionSubLayout->addWidget(precisionSpinBox);
    precisionLayout->addLayout(precisionSubLayout);

    precisionGroupBox->setLayout(precisionLayout);

    return precisionGroupBox;
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox "Farbschema".
//
QGroupBox* ControlWindow::createColorGroupBox()
{
    colorGroupBox = new QGroupBox(tr("Farbschema"), this);

    rgbRadioButton = new QRadioButton(tr("Gesamtes Farbspektrum"), this);
    grayRadioButton = new QRadioButton(tr("Graustufen"), this);
    bwRadioButton = new QRadioButton(tr("Schwarz und Weiss"), this);

    rgbRadioButton->setChecked(true);

    colorLayout = new QVBoxLayout;

    colorLayout->addWidget(rgbRadioButton);
    colorLayout->addWidget(grayRadioButton);
    colorLayout->addWidget(bwRadioButton);

    colorGroupBox->setLayout(colorLayout);

    return colorGroupBox;
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox "Algorithmus".
//
QGroupBox* ControlWindow::createAlgorithmGroupBox()
{
    algorithmGroupBox = new QGroupBox(tr("Algorithmus"), this);

    optimizedCheckBox = new QCheckBox(tr("Optimierte Variante verwenden"), this);
    optimizedCheckBox->setChecked(false);

    algorithmLayout = new QHBoxLayout;

    algorithmLayout->addWidget(optimizedCheckBox);

    algorithmGroupBox->setLayout(algorithmLayout);

    return algorithmGroupBox;
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox zum Wiedeherstellen der Standardwerte.
//
QGroupBox* ControlWindow::createRestoreGroupBox()
{
    restoreGroupBox = new QGroupBox("", this);

    restoreDefaultsButton = new QPushButton(tr("Standardwerte wiederherstellen"), this);

    restoreLayout = new QVBoxLayout;

    restoreLayout->addWidget(restoreDefaultsButton);

    restoreGroupBox->setLayout(restoreLayout);

    return restoreGroupBox;
}

//
// Einrichten saemtlicher UI-Elemente der GroupBox mit minimalistischen Statusinformationen.
//
QGroupBox* ControlWindow::createInformationGroupBox()
{
    informationGroupBox = new QGroupBox("", this);

    informationLabel = new QLabel(tr("Mandelbrot-Menge wird angezeigt"), this);
    informationLabel->setAlignment(Qt::AlignHCenter);

    informationLayout = new QVBoxLayout;

    informationLayout->addWidget(informationLabel);

    informationGroupBox->setLayout(informationLayout);

    return informationGroupBox;
}
