#ifndef _CONTROLWINDOW_HPP_
#define _CONTROLWINDOW_HPP_

#include <QWidget>
#include <QLayout>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QIcon>
#include <QLabel>
#include <QPushButton>
#include <QGroupBox>
#include <QSpinBox>
#include <QDoubleSpinBox>
#include <QRadioButton>
#include <QCheckBox>
#include <QCloseEvent>

/**
  * Anlegen und Anordnen aller UI-Elemente des Fensters "Einstellungen".
  *
  * @author Marcell Dietl
  */
class ControlWindow : public QWidget
{
    Q_OBJECT
public:
    /**
      * Erzeugt das ControlWindow ("Einstellungen").
      *
      * @param parent Stets 0, um das Widgets als Window zu definieren.
      */
    ControlWindow(QWidget* parent = 0);

    /** Mandelbrot-Menge berechnen und anzeigen. */
    QPushButton* renderMandelbrotButton;
    /** Zus&auml;tzliche Informationen zur Mandelbrot-Menge einblenden. */
    QPushButton* helpMandelbrotButton;
    /** Mandelbar-Menge berechnen und anzeigen. */
    QPushButton* renderMandelbarButton;
    /** Zus&auml;tzliche Informationen zur Mandelbar-Menge einblenden. */
    QPushButton* helpMandelbarButton;
    /** Multibrot-Menge berechnen und anzeigen. */
    QPushButton* renderMultibrotButton;
    /** Zus&auml;tzliche Informationen zur Multibrot-Menge einblenden. */
    QPushButton* helpMultibrotButton;
    /** Julia-Menge berechnen und anzeigen. */
    QPushButton* renderJuliaButton;
    /** Zus&auml;tzliche Informationen zur Julia-Menge einblenden. */
    QPushButton* helpJuliaButton;
    /** Zur&uuml;cksetzen aller Werte und Anzeigen der Mandelbrot-Menge. */
    QPushButton* restoreDefaultsButton;
    /** X-Wert des Mittelpunktes manuell einstellen. */
    QDoubleSpinBox* xSpinBox;
    /** Y-Wert des Mittelpunktes manuell einstellen. */
    QDoubleSpinBox* ySpinBox;
    /** Zoomfaktor, mit dem das Fraktal angezeigt werden soll. */
    QSpinBox* zoomSpinBox;
    /** Genauigkeit des Escape-Time-Algorithmus verbessern. */
    QSpinBox* precisionSpinBox;
    /** Minimalistische Statusanzeige zum Zustand der Anwendung. */
    QLabel* informationLabel;
    /** Setzen des Farbschemas "Gesamtes Farbspektrum". */
    QRadioButton* rgbRadioButton;
    /** Setzen des Farbschemas "Graustufen". */
    QRadioButton* grayRadioButton;
    /** Setzen des Farbschemas "Schwarz und Wei&szlig;". */
    QRadioButton* bwRadioButton;
    /** Festlegen, ob der optimierte Algorithmus verwendet werden soll. */
    QCheckBox* optimizedCheckBox;
    /** Exponent der Multibrot-Menge w&auml;hlen. */
    QDoubleSpinBox* multibrotSpinBox;
    /** Realteil des Parameters der Julia-Menge w&auml;hlen. */
    QDoubleSpinBox* juliaRealSpinBox;
    /** Imagin&auml;rteil des Parameters der Julia-Menge w&auml;hlen. */
    QDoubleSpinBox* juliaImagSpinBox;
protected:
    /**
      * &Uuml;berschreiben der Methode zum Schlie&szlig;en des Fensters.
      * Emittiert ein Signal zum Beenden der gesamten Anwendung.
      *
      * @param event Das abgefangene Ereignis.
      */
    void closeEvent(QCloseEvent *event);
signals:
    /** Alle Fenster der Anwendung und die Anwendung selbst schlie&szlig;en. */
    void closeAllWindows();
private:
    QGroupBox* createMandelbrotGroupBox();
    QGroupBox* createMandelbarGroupBox();
    QGroupBox* createMultibrotGroupBox();
    QGroupBox* createJuliaGroupBox();
    QGroupBox* createPositionGroupBox();
    QGroupBox* createZoomGroupBox();
    QGroupBox* createPrecisionGroupBox();
    QGroupBox* createColorGroupBox();
    QGroupBox* createAlgorithmGroupBox();
    QGroupBox* createRestoreGroupBox();
    QGroupBox* createInformationGroupBox();

    QGroupBox* mandelbrotGroupBox;
    QGroupBox* mandelbarGroupBox;
    QGroupBox* multibrotGroupBox;
    QGroupBox* juliaGroupBox;
    QGroupBox* positionGroupBox;
    QGroupBox* zoomGroupBox;
    QGroupBox* precisionGroupBox;
    QGroupBox* colorGroupBox;
    QGroupBox* algorithmGroupBox;
    QGroupBox* restoreGroupBox;
    QGroupBox* informationGroupBox;
    QLabel* exponentRangeLabel;
    QLabel* exponentParamLabel;
    QLabel* realRangeLabel;
    QLabel* realParamLabel;
    QLabel* imagRangeLabel;
    QLabel* imagParamLabel;
    QLabel* xRangeLabel;
    QLabel* xAxisLabel;
    QLabel* yRangeLabel;
    QLabel* yAxisLabel;
    QLabel* zoomRangeLabel;
    QLabel* precisionRangeLabel;
    QLayout* layout;
    QVBoxLayout* mandelbrotLayout;
    QHBoxLayout* mandelbrotSubLayout;
    QVBoxLayout* mandelbarLayout;
    QHBoxLayout* mandelbarSubLayout;
    QVBoxLayout* multibrotLayout;
    QHBoxLayout* multibrotSubLayout1;
    QHBoxLayout* multibrotSubLayout2;
    QVBoxLayout* juliaLayout;
    QHBoxLayout* juliaSubLayout1;
    QHBoxLayout* juliaSubLayout2;
    QHBoxLayout* juliaSubLayout3;
    QVBoxLayout* positionLayout;
    QHBoxLayout* positionSubLayout1;
    QHBoxLayout* positionSubLayout2;
    QVBoxLayout* zoomLayout;
    QHBoxLayout* zoomSubLayout;
    QVBoxLayout* precisionLayout;
    QHBoxLayout* precisionSubLayout;
    QVBoxLayout* colorLayout;
    QHBoxLayout* algorithmLayout;
    QVBoxLayout* restoreLayout;
    QVBoxLayout* informationLayout;
};

#endif // _CONTROLWINDOW_HPP_
