#ifndef _CONTROLWINDOWIMPL_HPP_
#define _CONTROLWINDOWIMPL_HPP_

#include <QApplication>
#include <QObject>
#include <string>
#include <complex>

#include "controlwindow.hpp"
#include "mainwindow.hpp"
#include "complexfractalrenderer.hpp"
#include "mandelbrotmenge.hpp"
#include "mandelbarmenge.hpp"
#include "juliamenge.hpp"
#include "multibrotmenge.hpp"
#include "rgbcolormapper.hpp"
#include "graycolormapper.hpp"
#include "bwcolormapper.hpp"

/**
  * Implementierungen der Kn&ouml;pfe des Fensters "Einstellungen".
  *
  * @author Marcell Dietl
  */
class ControlWindowImpl : public QObject
{
    Q_OBJECT
public:
    /**
      * Erzeugt die ControlWindowImpl(ementation).
      *
      * @param mw Pointer auf das Hauptfenster "Fraktale".
      * @param cw Pointer auf das Fenster "Einstellungen".
      */
    ControlWindowImpl(MainWindow* mw, ControlWindow* cw);

    /**
      * Erzeugen des Fraktals anhand der im Fenster eingestellten Werte.
      *
      * @param fractal Typ des Fraktals, das berechnet werden soll.
      * @param fractalName String-Repr&auml;sentierung des Fraktals.
      * @param size Multiplikationsfaktor zum Erzeugen gr&ouml;&szlig;erer Bilder.
      * @param show Angabe, ob der Methodenaufruf das angezeigte Fraktal veraendert.
      * @return Das gerenderte Fraktal.
      */
    QImage renderFractal(ComplexFractal* fractal, std::string fractalName = "", bool show = true, int size = 1);
signals:
    /**
      * &Uuml;bertragen des Fraktals an das MainWindow.
      *
      * @param newFractal Das gerenderte Fraktal.
      */
    void renderedFractal(QImage newFractal);
public slots:
    /** Berechnen und &Uuml;bertragen des Bildes der Mandelbrot-Menge. */
    void renderMandelbrotImpl();
    /** Berechnen und &Uuml;bertragen des Bildes der Mandelbar-Menge. */
    void renderMandelbarImpl();
    /** Berechnen und &Uuml;bertragen des Bildes der Multibrot-Menge. */
    void renderMultibrotImpl();
    /** Berechnen und &Uuml;bertragen des Bildes der Julia-Menge. */
    void renderJuliaImpl();
    /** Zur&uuml;cksetzen aller Werte und Anzeigen der Mandelbrot-Menge. */
    void restoreDefaultsImpl();
    /**
      * Anpassen der Koordinaten des Mittelpunktes nach Mausklick im Fraktal.
      *
      * @param x Wert auf der realen Achse des angeklickten Pixels.
      * @param y Wert auf der imagin&auml;ren Achse des angeklickten Pixels.
      */
    void imageLabelClicked(int x, int y);
private:
    MainWindow* mw;
    ControlWindow* cw;

    QImage newFractal;
    Mandelbrotmenge* mandelbrot;
    Mandelbarmenge* mandelbar;
    Multibrotmenge* multibrot;
    Juliamenge* julia;
    ColorMapper* mapper;
    ComplexFractalRenderer* fractalRenderer;
};

#endif // _CONTROLWINDOWIMPL_HPP_
