#include <QApplication>
#include <QObject>
#include <QTranslator>
#include <QLocale>

#include "controlwindow.hpp"
#include "controlwindowimpl.hpp"
#include "mainwindow.hpp"
#include "mainwindowimpl.hpp"

/** @mainpage
  *
  * <b>Entwickelt von Marcell Dietl (www.marcell-dietl.de) im Januar 2012
  * als Abschlussprojekt des Moduls "Programmieren 3 AI" (Java und C++)
  * des Studiengangs "Allgemeine Informatik" der Hochschule RheinMain
  * am Standort Wiesbaden in Hessen.</b>
  *
  * <hr>
  *
  * <b>Folgende Fraktale k&ouml;nnen interaktiv erkundert werden: Die
  * <a href="http://en.wikipedia.org/wiki/Mandelbrot_set" target="_blank">Mandelbrot-</a>,
  * <a href="http://en.wikipedia.org/wiki/Tricorn_%28mathematics%29" target="_blank">Mandelbar-</a>,
  * <a href="http://en.wikipedia.org/wiki/Multibrot_set" target="_blank">Multibrot-</a> und
  * <a href="http://en.wikipedia.org/wiki/Julia_set" target="_blank">Julia-Menge</a>.</b>
  *
  * <hr>
  *
  * @image html startup.png
  */

int main(int argc, char* argv[])
{
    QApplication app(argc, argv);

    //
    // Laden der (de)utschen oder (en)glischen QM-Datei.
    // Als Fallback-Sprache dienen die ASCII-Strings innerhalb der Anwendung.
    //
    QTranslator translator;
    translator.load("Fraktale_" + QLocale::system().name(), ":/");
    app.installTranslator(&translator);

    //
    // Initialisieren der Fenster und ihrer jeweiligen Implementierungen.
    //
    ControlWindow* control = new ControlWindow();
    MainWindow* main = new MainWindow();
    ControlWindowImpl* controlImpl = new ControlWindowImpl(main, control);
    MainWindowImpl* mainImpl = new MainWindowImpl(main, control, controlImpl);

    //
    // Verknuepfen des anklickbaren Fraktal-Bildes mit dem Einstellungs-Fenster.
    //
    QObject::connect(main->fractalLabel, SIGNAL(clicked(int,int)), controlImpl, SLOT(imageLabelClicked(int,int)));

    //
    // Wenn eines der Fenster geschlossen wird, beendet sich die Anwendung.
    //
    QObject::connect(control, SIGNAL(closeAllWindows()), QApplication::instance(), SLOT(quit()));
    QObject::connect(main, SIGNAL(closeAllWindows()), QApplication::instance(), SLOT(quit()));

    //
    // Verknuepfen der Menue-Eintraege mit ihren jeweiligen Implementierungen.
    //
    QObject::connect(main->saveAsLowAction, SIGNAL(triggered()), mainImpl, SLOT(saveAsLow()));
    QObject::connect(main->saveAsMediumAction, SIGNAL(triggered()), mainImpl, SLOT(saveAsMedium()));
    QObject::connect(main->saveAsHighAction, SIGNAL(triggered()), mainImpl, SLOT(saveAsHigh()));
    QObject::connect(main->quitAction, SIGNAL(triggered()), QApplication::instance(), SLOT(quit()));
    QObject::connect(main->mandelbrotAction, SIGNAL(triggered()), mainImpl, SLOT(mandelbrot()));
    QObject::connect(main->mandelbarAction, SIGNAL(triggered()), mainImpl, SLOT(mandelbar()));
    QObject::connect(main->multibrotAction, SIGNAL(triggered()), mainImpl, SLOT(multibrot()));
    QObject::connect(main->juliaAction, SIGNAL(triggered()), mainImpl, SLOT(julia()));
    QObject::connect(main->aboutAction, SIGNAL(triggered()), mainImpl, SLOT(about()));

    //
    // Verknuepfen der Knoepfe im Einstellungs-Fenster mit ihren jeweiligen Implementierungen.
    //
    QObject::connect(control->renderMandelbrotButton, SIGNAL(clicked()), controlImpl, SLOT(renderMandelbrotImpl()));
    QObject::connect(controlImpl, SIGNAL(renderedFractal(QImage)), main, SLOT(setFractal(QImage)));
    QObject::connect(control->helpMandelbrotButton, SIGNAL(clicked()), mainImpl, SLOT(mandelbrot()));

    QObject::connect(control->renderMandelbarButton, SIGNAL(clicked()), controlImpl, SLOT(renderMandelbarImpl()));
    QObject::connect(controlImpl, SIGNAL(renderedFractal(QImage)), main, SLOT(setFractal(QImage)));
    QObject::connect(control->helpMandelbarButton, SIGNAL(clicked()), mainImpl, SLOT(mandelbar()));

    QObject::connect(control->renderMultibrotButton, SIGNAL(clicked()), controlImpl, SLOT(renderMultibrotImpl()));
    QObject::connect(controlImpl, SIGNAL(renderedFractal(QImage)), main, SLOT(setFractal(QImage)));
    QObject::connect(control->helpMultibrotButton, SIGNAL(clicked()), mainImpl, SLOT(multibrot()));

    QObject::connect(control->renderJuliaButton, SIGNAL(clicked()), controlImpl, SLOT(renderJuliaImpl()));
    QObject::connect(controlImpl, SIGNAL(renderedFractal(QImage)), main, SLOT(setFractal(QImage)));
    QObject::connect(control->helpJuliaButton, SIGNAL(clicked()), mainImpl, SLOT(julia()));

    QObject::connect(control->restoreDefaultsButton, SIGNAL(clicked()), controlImpl, SLOT(restoreDefaultsImpl()));
    QObject::connect(controlImpl, SIGNAL(renderedFractal(QImage)), main, SLOT(setFractal(QImage)));

    //
    // Los geht's!
    //
    control->show();
    main->show();

    return app.exec();
}
