#include "mainwindow.hpp"

MainWindow::MainWindow(QWidget *parent)
: QMainWindow(parent)
{
    centralWidget = new QWidget;

    //
    // Das MainWindow ist nun Eigentuemer des Widgets.
    //
    setCentralWidget(centralWidget);

    createFileMenu();
    createHelpMenu();

    layout = new QVBoxLayout;

    fractalLabel = new ImageLabel(this);

    //
    // Das Fraktal-Bild passt sich der Groesse des MainWindow an.
    //
    fractalLabel->setScaledContents(true);

    setDefaultFractal();

    layout->addWidget(fractalLabel);

    centralWidget->setLayout(layout);
    setWindowTitle(tr("Fraktale"));
    setWindowIcon(QIcon(":/hsrmlogo.png"));
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    //
    // Wenn eines der Fenster geschlossen wird, beendet sich die Anwendung.
    //
    event->accept();
    emit closeAllWindows();
}

//
// Einrichten saemtlicher Menue-Eintraege des "Datei"-Menues.
//
void MainWindow::createFileMenu()
{
    fileMenu = menuBar()->addMenu(tr("Datei"));

    saveAsMenu = fileMenu->addMenu(tr("Bild speichern unter..."));

    //
    // Einrichten eines Sub-Menue des "Bild speichern unter..."-Menues, in dem
    // die Aufloesung des zu speichernden Fraktal-Bildes gewaehlt werden kann.
    //
    saveAsLowAction = new QAction(tr("Niedrige Aufloesung (800 x 600)"), this);
    saveAsMediumAction = new QAction(tr("Mittlere Aufloesung (1600 x 1200)"), this);
    saveAsHighAction = new QAction(tr("Hohe Aufloesung (3200 x 2400)"), this);

    saveAsMenu->addAction(saveAsLowAction);
    saveAsMenu->addAction(saveAsMediumAction);
    saveAsMenu->addAction(saveAsHighAction);

    quitAction = new QAction(tr("Beenden"), this);

    fileMenu->addSeparator();
    fileMenu->addAction(quitAction);
}

//
// Einrichten saemtlicher Menue-Eintraege des "Hilfe"-Menues.
//
void MainWindow::createHelpMenu()
{
    helpMenu = menuBar()->addMenu(tr("Hilfe"));

    mandelbrotAction = new QAction(tr("Mandelbrot-Menge"), this);
    mandelbarAction = new QAction(tr("Mandelbar-Menge"), this);
    multibrotAction = new QAction(tr("Multibrot-Menge"), this);
    juliaAction = new QAction(tr("Julia-Menge"), this);
    aboutAction = new QAction(tr("Ueber Fraktale"), this);

    helpMenu->addAction(mandelbrotAction);
    helpMenu->addAction(mandelbarAction);
    helpMenu->addAction(multibrotAction);
    helpMenu->addAction(juliaAction);
    helpMenu->addSeparator();
    helpMenu->addAction(aboutAction);
}

//
// Setzen eines neuen Fraktal-Bildes.
//
void MainWindow::setFractal(QImage newFractal)
{
    fractalImage = newFractal;
    fractalLabel->setPixmap(QPixmap::fromImage(newFractal));
}

//
// Setzen das standardmaessigen Fraktal-Bildes.
//
void MainWindow::setDefaultFractal()
{
    fractalLabel->topLeft.real() = -2.0;
    fractalLabel->topLeft.imag() = 1.5;
    fractalLabel->name = "mandelbrot";

    setFractal(QImage(":/default.png"));
}
