#ifndef _MAINWINDOW_HPP_
#define _MAINWINDOW_HPP_

#include <QMainWindow>
#include <QWidget>
#include <QLayout>
#include <QVBoxLayout>
#include <QIcon>
#include <QMenuBar>
#include <QMenu>
#include <QImage>
#include <QAction>
#include <QCloseEvent>

#include "imagelabel.hpp"

/**
  * Anlegen und Anordnen aller UI-Elemente des Hauptfensters "Fraktale".
  *
  * @author Marcell Dietl
  */
class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    /**
      * Erzeugt das MainWindow ("Fraktale").
      *
      * @param parent Stets 0, um das Widgets als Window zu definieren.
      */
    MainWindow(QWidget *parent = 0);

    /** Zeigt das Standard-Fraktal, die Mandelbrot-Menge, an. */
    void setDefaultFractal();

    /** Das Bild des aktuell angezeigten Fraktals. */
    QImage fractalImage;
    /** Das Label des aktuell angezeigten Fraktals. */
    ImageLabel* fractalLabel;
    /** Men&uuml;-Eintrag "Niedrige Aufl&ouml;sung (800 x 600)". */
    QAction* saveAsLowAction;
    /** Men&uuml;-Eintrag "Mittlere Aufl&ouml;sung (1600 x 1200)". */
    QAction* saveAsMediumAction;
    /** Men&uuml;-Eintrag "Hohe Aufl&ouml;sung (3200 x 2400)". */
    QAction* saveAsHighAction;
    /** Men&uuml;-Eintrag "Beenden". */
    QAction* quitAction;
    /** Men&uuml;-Eintrag "Mandelbrot-Menge". */
    QAction* mandelbrotAction;
    /** Men&uuml;-Eintrag "Mandelbar-Menge". */
    QAction* mandelbarAction;
    /** Men&uuml;-Eintrag "Multibrot-Menge". */
    QAction* multibrotAction;
    /** Men&uuml;-Eintrag "Julia-Menge". */
    QAction* juliaAction;
    /** Men&uuml;-Eintrag "&Uuml;ber Fraktale". */
    QAction* aboutAction;
protected:
    /**
      * &Uuml;berschreiben der Methode zum Schlie&szlig;en des Fensters.
      * Emittiert ein Signal zum Beenden der gesamten Anwendung.
      *
      * @param event Das abgefangene Ereignis.
      */
    void closeEvent(QCloseEvent *event);
signals:
    /** Alle Fenster der Anwendung und die Anwendung selbst schlie&szlig;en. */
    void closeAllWindows();
public slots:
    /**
      * Ver&auml;ndert das aktuell angezeigte Fraktals.
      *
      * @param newFractal Das neue Fraktal.
      */
    void setFractal(QImage newFractal);
private:
    void createFileMenu();
    void createHelpMenu();

    QLayout* layout;
    QMenu* fileMenu;
    QMenu* saveAsMenu;
    QMenu* helpMenu;
    QWidget* centralWidget;
};

#endif // _MAINWINDOW_HPP_
