#include "multibrotmenge.hpp"

//
// Berechnet den Wert fuer einen bestimmten Punkt in der gaussschen Zahlenebene mit
// Hilfe des Escape-Time-Algorithmus. Wurde der Parameter optimized auf true gesetzt,
// wird die optimierte Variante mit gleichmaessigerer Einfaerbung verwendet.
//
double Multibrotmenge::calculate(const std::complex<double>& s, int precision, bool optimized) const
{
    int t = 4;
    std::complex<double> z(0);
    int n = 0;

    while ((std::abs(z) < t) && (n < precision))
    {
        //
        // Vergleich zur Berechnungsvorschrift der Mandelbrot-Menge:
        // -------------------------------------------------------------
        // Der reelle Exponent point ist frei waehlbar.
        //
        z = pow(z, point) + s;
        n = n + 1;
    }

    //
    // Entscheidung, ob der klassische oder optimierte Algorithmus
    // zur Berechnung des Rueckgabewertes zum Einsatz kommt.
    //
    if (optimized)
    {
        double res = n - std::log((std::log(std::abs(z)))) / std::log(point.real());
               res = (std::isnan(res)) ? 0 : (1.0 - (1.0 / precision) * res);

           return (res > 1 || res < 0) ? 0 : res;
    }
    else
    {
        return 1.0 - (1.0 / precision) * n;
    }
}

//
// Getter- und Setter-Methoden der Schnittstelle ParametrizedComplexFractal.
//
std::complex<double> Multibrotmenge::getParam() const
{
    return point;
}

void Multibrotmenge::setParam(const std::complex<double>& p)
{
    point = p;
}
