#ifndef _MULTIBROTMENGE_HPP_
#define _MULTIBROTMENGE_HPP_

#include "parametrizedcomplexfractal.hpp"

/**
  * Multibrot-Menge.
  *
  * @author Marcell Dietl
  */
class Multibrotmenge : public ParametrizedComplexFractal
{
public:
    /**
      * Berechnet den Wert f&uuml;r einen bestimmten Punkt.
      *
      * @param s Punkt in der komplexen Zahlenebene, f&uuml;r den die Berechnung durchgef&uuml;hrt werden soll.
      * @param precision Genauigkeit mit der die einzelnen Pixel gerendert werden soll.
      * @param optimized Aktivieren des optimierten Escape-Time-Algorithmus.
      * @return Ergebnis der Berechnung, liegt im Intervall [0; 1].
      */
    virtual double calculate(const std::complex<double>& s, int precision, bool optimized) const;
    /**
      * Gibt den aktuellen Wert des Parameters zur&uuml;ck.
      *
      * @return Aktueller Wert des Parameters.
      */
    virtual std::complex<double> getParam() const;
    /**
      * Setzt den Parameter auf einen neuen Wert.
      *
      * @param p Wert, auf den der Parameter gesetzt werden soll.
      */
    virtual void setParam(const std::complex<double>& p);
private:
    std::complex<double> point;
};

#endif // _MULTIBROTMENGE_HPP_
